package client.controller;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;

import client.model.CamModel;
import client.view.panel.CamControlPanel;

import com.centralnexus.input.*;

public class CamController {

	private CamModel.CameraDirection cam_state = CamModel.CameraDirection.STOP_ALL;
	
	private Joystick joy;
	
	public CamController(){
		//Register in the service registry in order to avoid multiple instances?
		
		
		try {
			joy = Joystick.createInstance();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	//Add listeners
	public void addListeners(final CamControlPanel viewer, final CamModel model){
		//Initializing buttonListeners
		model.upMouseListener = new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent e) { moveCam(model, CamModel.CameraDirection.UP, false); }		
			@Override
			public void mousePressed(MouseEvent e) { moveCam(model, CamModel.CameraDirection.UP, true); }		
			@Override
			public void mouseExited(MouseEvent e) {/*	moveCam(model, CamModel.CameraDirection.UP, false); */}			
			@Override
			public void mouseEntered(MouseEvent e) {}		
			@Override
			public void mouseClicked(MouseEvent e) {}
			
		};
		
		model.downMouseListener = new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent e) { moveCam(model, CamModel.CameraDirection.DOWN, false); }		
			@Override
			public void mousePressed(MouseEvent e) { moveCam(model, CamModel.CameraDirection.DOWN, true);}		
			@Override
			public void mouseExited(MouseEvent e) {/*	moveCam(model, CamModel.CameraDirection.DOWN, false); */}			
			@Override
			public void mouseEntered(MouseEvent e) {}		
			@Override
			public void mouseClicked(MouseEvent e) {}
			
		};
		
		model.leftMouseListener = new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent e) { moveCam(model, CamModel.CameraDirection.LEFT, false); }		
			@Override
			public void mousePressed(MouseEvent e) { moveCam(model, CamModel.CameraDirection.LEFT, true); }		
			@Override
			public void mouseExited(MouseEvent e) {/*	moveCam(model, CamModel.CameraDirection.LEFT, false); */}			
			@Override
			public void mouseEntered(MouseEvent e) {}		
			@Override
			public void mouseClicked(MouseEvent e) {}
			
		};
		
		model.rightMouseListener = new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent e) { moveCam(model, CamModel.CameraDirection.RIGHT, false); }		
			@Override
			public void mousePressed(MouseEvent e) {moveCam(model, CamModel.CameraDirection.RIGHT, true); }		
			@Override
			public void mouseExited(MouseEvent e) {/*	moveCam(model, CamModel.CameraDirection.RIGHT, false); */}			
			@Override
			public void mouseEntered(MouseEvent e) {}		
			@Override
			public void mouseClicked(MouseEvent e) {}
			
		};
		
		model.stopAllMouseListener = new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent e) {moveCam(model, CamModel.CameraDirection.STOP_ALL, false); }		
			@Override
			public void mousePressed(MouseEvent e) { moveCam(model, CamModel.CameraDirection.STOP_ALL, true); }		
			@Override
			public void mouseExited(MouseEvent e) {/*	moveCam(model, CamModel.CameraDirection.STOP_ALL, false); */}			
			@Override
			public void mouseEntered(MouseEvent e) {}		
			@Override
			public void mouseClicked(MouseEvent e) {}
			
		};
		
		model.joyListener = new JoystickListener() {

			@Override
			public void joystickAxisChanged(Joystick arg0) {
				// TODO Auto-generated method stub
				double z = arg0.getZ();
				double r = arg0.getR();
		
				switch(cam_state){
				case STOP_ALL:
					if(z > 0.3) {
						moveCam(model, CamModel.CameraDirection.DOWN, true);
						cam_state = CamModel.CameraDirection.DOWN;
					}
					if(z < -0.3) {
						moveCam(model, CamModel.CameraDirection.UP, true);
						cam_state = CamModel.CameraDirection.UP;
					}
					if(r > 0.3) {
						moveCam(model, CamModel.CameraDirection.RIGHT, true);
						cam_state = CamModel.CameraDirection.RIGHT;
					}
					if(r < -0.3) {
						moveCam(model, CamModel.CameraDirection.LEFT, true);
						cam_state = CamModel.CameraDirection.LEFT;
					}		
					break;
				case RIGHT:
					if(r < 0.3) {
						moveCam(model, CamModel.CameraDirection.RIGHT, false);
						cam_state = CamModel.CameraDirection.STOP_ALL;
					}
					break;
				case LEFT:
					if(r > -0.3) {
						moveCam(model, CamModel.CameraDirection.LEFT, false);
						cam_state = CamModel.CameraDirection.STOP_ALL;
					}
					break;
				case UP:
					if(z > -0.3) {
						moveCam(model, CamModel.CameraDirection.UP, false);
						cam_state = CamModel.CameraDirection.STOP_ALL;
					}
					break;
				case DOWN:
					if(z < 0.3) {
						moveCam(model, CamModel.CameraDirection.DOWN, false);
						cam_state = CamModel.CameraDirection.STOP_ALL;
					}
					break;
				}
			}
			
			@Override
			public void joystickButtonChanged(Joystick arg0) {
				// TODO Auto-generated method stub
				
			}
			
		};
		
		model.keyListener = new KeyListener() {

			@Override
			public void keyPressed(KeyEvent arg0) {
				// TODO Auto-generated method stub
				System.out.println("PRESSED:" + arg0.toString());

			}

			@Override
			public void keyReleased(KeyEvent arg0) {
				// TODO Auto-generated method stub
				System.out.println("RELEASED:" + arg0.toString());
			}

			@Override
			public void keyTyped(KeyEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		};
		
		
		joy.addJoystickListener(model.joyListener);
		
		viewer.addKeyListener(model.keyListener);
		
		//Adding buttonListeners
		viewer.getUpBtn().addMouseListener(model.getUpMouseListener());
		viewer.getDownBtn().addMouseListener(model.getDownMouseListener());
		viewer.getLeftBtn().addMouseListener(model.getLeftMouseListener());
		viewer.getRightBtn().addMouseListener(model.getRightMouseListener());
		viewer.getStopAllBtn().addMouseListener(model.getStopAllMouseListener());
		
	}
	
	//Remove listeners
	public void removeListeners(final CamControlPanel viewer, final CamModel model){
		//Remove all listeners (free memory)
		viewer.getUpBtn().removeMouseListener(model.getUpMouseListener());
		viewer.getDownBtn().removeMouseListener(model.getDownMouseListener());
		viewer.getLeftBtn().removeMouseListener(model.getLeftMouseListener());
		viewer.getRightBtn().removeMouseListener(model.getRightMouseListener());
		viewer.getStopAllBtn().removeMouseListener(model.getStopAllMouseListener());
	}

	//Controller functions
	
	public void moveCam(CamModel model, CamModel.CameraDirection command, boolean enable){
		//Checking if new state: Not a pushed button , 
//FIXME		if((!enable && command==CamModel.STOP_ALL) && !model.isLastCommandEnable() && command==CamModel.STOP_ALL) return;		
	
		//Updating model
		model.setLastCommand(command, enable);
		
		//Telling driver what to do
		switch (command) {
		case UP:
			if(enable) model.getCameraDriver().goUp();
			else model.getCameraDriver().stopUp();
			break;
			
		case DOWN:
			if(enable) model.getCameraDriver().goDown();
			else model.getCameraDriver().stopDown();
			break;
		
		case LEFT:
			if(enable) model.getCameraDriver().goLeft();
			else model.getCameraDriver().stopLeft();
			break;
		
		case RIGHT:
			if(enable) model.getCameraDriver().goRight();
			else model.getCameraDriver().stopRight();
			break;
			
		case STOP_ALL:
			model.getCameraDriver().stopAll();
			break;
		
		default:
			model.getCameraDriver().stopAll();
			break;
		}
	}
	
}
