package client.controller;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import client.drivers.VehicleDriver.VehicleCommand;
import client.model.VehicleModel;
import client.view.panel.VehicleControlPanel;
import com.centralnexus.input.*;





public class VehicleController {
	
	private Joystick joy;
	
	public VehicleController(){
		//Register in the service registry in order to avoid multiple instances?
		
		
		try {
			joy = Joystick.createInstance();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		

	}
	
	//Add listeners
	public void addListeners(final VehicleControlPanel viewer, final VehicleModel model){
		//Initializing buttonListeners
		model.fastFwdMouseListener = new MouseListener() {
			@Override
			public void mouseReleased(MouseEvent arg0) { moveVehicle(model, VehicleCommand.STOP); }
			@Override
			public void mousePressed(MouseEvent arg0) { moveVehicle(model, VehicleCommand.FFWD); }
			@Override
			public void mouseExited(MouseEvent arg0) {/* moveVehicle(model, VehicleCommand.STOP); */}
			@Override
			public void mouseEntered(MouseEvent arg0) {}
			@Override
			public void mouseClicked(MouseEvent arg0) {}
		};
		
		model.fwdMouseListener = new MouseListener() {
			@Override
			public void mouseReleased(MouseEvent arg0) { moveVehicle(model, VehicleCommand.STOP); }
			@Override
			public void mousePressed(MouseEvent arg0) { moveVehicle(model, VehicleCommand.FWD); }
			@Override
			public void mouseExited(MouseEvent arg0) {/* moveVehicle(model, VehicleCommand.STOP); */}
			@Override
			public void mouseEntered(MouseEvent arg0) {}
			@Override
			public void mouseClicked(MouseEvent arg0) {}
		};
		
		model.revMouseListener = new MouseListener() {
			@Override
			public void mouseReleased(MouseEvent arg0) { moveVehicle(model, VehicleCommand.STOP); }
			@Override
			public void mousePressed(MouseEvent arg0) { moveVehicle(model, VehicleCommand.REV); }
			@Override
			public void mouseExited(MouseEvent arg0) {/* moveVehicle(model, VehicleCommand.STOP); */}
			@Override
			public void mouseEntered(MouseEvent arg0) {}
			@Override
			public void mouseClicked(MouseEvent arg0) {}
		};
		
		model.fastRevMouseListener = new MouseListener() {
			@Override
			public void mouseReleased(MouseEvent arg0) { moveVehicle(model, VehicleCommand.STOP); }
			@Override
			public void mousePressed(MouseEvent arg0) { moveVehicle(model, VehicleCommand.RREV); }
			@Override
			public void mouseExited(MouseEvent arg0) {/* moveVehicle(model, VehicleCommand.STOP); */}
			@Override
			public void mouseEntered(MouseEvent arg0) {}
			@Override
			public void mouseClicked(MouseEvent arg0) {}
		};
		
		
		model.joyListener = new JoystickListener() {

			@Override
			public void joystickAxisChanged(Joystick arg0) {
				// TODO Auto-generated method stub
				/*System.out.println(arg0.toString());
		        System.out.println(Double.toString(arg0.getY()));
		        System.out.println(Double.toString(arg0.getZ()));
		        System.out.println(Double.toString(arg0.getR()));*/
				double y = arg0.getY();
				
				if(y < -0.6) moveVehicle(model, VehicleCommand.FFWD);
				if(y < -0.3 && y >= -0.6) moveVehicle(model, VehicleCommand.FWD);
				if(y < 0.3 && y >= -0.3) moveVehicle(model, VehicleCommand.STOP);
				if(y < 0.6 && y >= 0.3) moveVehicle(model, VehicleCommand.REV);
				if(y >= 0.6) moveVehicle(model, VehicleCommand.RREV);

				
				
			}

			@Override
			public void joystickButtonChanged(Joystick arg0) {
				// TODO Auto-generated method stub
				
			}
			
			
		};
		
		joy.addJoystickListener(model.joyListener);
		
	
		
		//Adding buttonListeners
		viewer.getFastForwardBtn().addMouseListener(model.getFastFwdMouseListener());
		viewer.getForwardBtn().addMouseListener(model.getFwdMouseListener());
		viewer.getReverseBtn().addMouseListener(model.getRevMouseListener());
		viewer.getFastReverseBtn().addMouseListener(model.getFastRevMouseListener());
	}
	
	//Remove listeners
	public void removeListeners(final VehicleControlPanel viewer, final VehicleModel model){
		//Remove all listeners (free memory)
		viewer.getForwardBtn().removeMouseListener(model.getFwdMouseListener());
		viewer.getReverseBtn().removeMouseListener(model.getRevMouseListener());
	}
	
	//Controller functions
	private void moveVehicle(VehicleModel model, VehicleCommand command){
		model.getVehicleDriver().sendVehicleCommand(command);
	}
	
}
