package client.model;

import java.awt.event.KeyListener;
import java.awt.event.MouseListener;

import client.ClientConstants;
import client.drivers.IPCamera.IPCameraDriver;

import com.centralnexus.input.*;

public class CamModel extends ModelPropertyChange {
	
	//Commands
	public static enum CameraDirection{ UP, DOWN, LEFT, RIGHT, STOP_ALL, UNKNOWN}
	
//	public static final int UP = 1;
//	public static final int DOWN = 2;
//	public static final int LEFT = 3;
//	public static final int RIGHT = 4;
//	public static final int STOP_ALL = 5;
	
	//Variables
	private CameraDirection lastCommandDirection;	//Used to prevent unnecessary network traffic
	private boolean lastCommandEnable;	//Used to prevent unnecessary network traffic
	public MouseListener upMouseListener;
	public MouseListener downMouseListener;
	public MouseListener leftMouseListener;
	public MouseListener rightMouseListener;
	public MouseListener stopAllMouseListener;
	
	public JoystickListener joyListener;
	public KeyListener keyListener;
	private final IPCameraDriver cameraDriver;
	
	public CamModel(){
		super();
		lastCommandDirection = CameraDirection.UNKNOWN;
		cameraDriver = new IPCameraDriver(ClientConstants.CAMERA_ADDRESS, ClientConstants.CAMERA_USERNAME, ClientConstants.CAMERA_PASSWORD);

	}
		
	//Getters & Setters
	
	
	
	public void setLastCommand(CameraDirection command, boolean enable) {
		//Setting new status
		CameraDirection lastDir = this.lastCommandDirection;
		boolean lastMode = this.lastCommandEnable;
		this.lastCommandDirection = command;
		this.lastCommandEnable = lastMode;

		//Firing property change
		changeSupport.firePropertyChange(ClientConstants.CAMERA_CONTROLLER_MOVEMENT_SET_DIR, lastDir, lastCommandDirection);
		changeSupport.firePropertyChange(ClientConstants.CAMERA_CONTROLLER_MOVEMENT_SET_MODE, lastMode, lastCommandEnable);
	}
	
	public CameraDirection getLastCommandDirection() {
		return lastCommandDirection;
	}

	public boolean isLastCommandEnable() {
		return lastCommandEnable;
	}

	public MouseListener getUpMouseListener() {
		return upMouseListener;
	}


	public MouseListener getDownMouseListener() {
		return downMouseListener;
	}


	public MouseListener getLeftMouseListener() {
		return leftMouseListener;
	}


	public MouseListener getRightMouseListener() {
		return rightMouseListener;
	}


	public MouseListener getStopAllMouseListener() {
		return stopAllMouseListener;
	}

	public IPCameraDriver getCameraDriver() {
		return cameraDriver;
	}


	
	
	
}
